@@ -426,6 +426,7 @@ void OnFile_RunExe() {
 		/* Add file filter */
 		GtkFileFilter *exefilter = gtk_file_filter_new ();
 		gtk_file_filter_add_pattern (exefilter, "*.exe");
+		gtk_file_filter_add_pattern (exefilter, "*.psx");
 		gtk_file_filter_set_name (exefilter, "Playstation Executable Files");
 		gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (file_chooser),
 									 exefilter);
@@ -1951,7 +1952,7 @@ void ScanPlugins(gchar* scandir) {
 		while ((ent = readdir(dir)) != NULL) {
 			filename = g_build_filename (scandir, ent->d_name, NULL);
 
-			if ((match(filename, ".*\.so$") == 0) && (match(filename, "cfg.*") == 0)) {
+			if ((match(filename, ".*\\.so$") == 0) && (match(filename, "cfg.*") == 0)) {
 
 				continue;	/* Skip this file */
 			} else {
