@@ -401,7 +401,7 @@ static void recReset() {
 	memset(recRAM, 0, 0x200000);
 	memset(recROM, 0, 0x080000);
 
-	x86Init();
+	//x86Init();
 	x86SetPtr(recMem);
 
 	branch = 0;
