@@ -25,7 +25,7 @@
  */
 
 #include "ix86.h"
-
+#if 0
 static s32 iCpuId(s32 cmd, u32 *regs) {
 	int flag;
 
@@ -104,6 +104,7 @@ void x86Init() {
 		}
 	}
 }
+#endif
 
 void x86SetPtr(char *ptr) {
 	x86Ptr = ptr;
