@@ -404,18 +404,27 @@ static void rec##f() { \
 
 static void recRecompile();
 
+#define RECRAM_POS (&psxMemWrite32 - (0x280000*sizeof(void*)) - 0xc000000)
+#define RECMEM_POS (RECRAM_POS - RECMEM_SIZE)
+
 static int recInit() {
 	int i;
 
 	psxRecLUT = (uptr**) malloc(0x010000 * sizeof(void*));
 
-	//recMem = (char*) malloc(RECMEM_SIZE + 0x1000);
-	recMem = mmap(0, RECMEM_SIZE + 0x1000,
+	recMem = mmap(RECMEM_POS,
+	//recMem = mmap(0,
+		RECMEM_SIZE,
 		PROT_EXEC | PROT_WRITE | PROT_READ, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
-	memset(recMem, 0, RECMEM_SIZE + 0x1000);
+	memset(recMem, 0, RECMEM_SIZE);
+
+	recRAM = mmap(RECRAM_POS,
+		0x280000*sizeof(void*),
+		PROT_WRITE | PROT_READ, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+	//recRAM = (uptr*) malloc(0x200000 * sizeof(void*));
+	//recROM = (uptr*) malloc(0x080000 * sizeof(void*));
+	recROM = &recRAM[0x200000];
 
-	recRAM = (uptr*) malloc(0x200000 * sizeof(void*));
-	recROM = (uptr*) malloc(0x080000 * sizeof(void*));
 	if (recRAM == NULL || recROM == NULL || recMem == NULL || psxRecLUT == NULL) {
 		SysMessage("Error allocating memory"); return -1;
 	}
@@ -449,9 +458,10 @@ static void recShutdown() {
 	if (recMem == NULL) return;
 	free(psxRecLUT);
 	//free(recMem);
-	munmap(recMem, RECMEM_SIZE + 0x1000);
-	free(recRAM);
-	free(recROM);
+	munmap(recMem, RECMEM_SIZE);
+	//free(recRAM);
+	munmap(recMem, 0x280000*sizeof(void*));
+	//free(recROM);
 	x86Shutdown();
 }
 
@@ -1380,7 +1390,9 @@ static void recLHU() {
 
 				//PUSHI  (addr);
 				MOV64ItoR(X86ARG1, addr);
-				CALLFunc  ((uptr)SPU_readRegister);
+				//CALLFunc  ((uptr)SPU_readRegister);
+				MOV64ItoR(RAX, (uptr)SPU_readRegister);
+				CALL64R(RAX);
 				MOVZX32R16toR(EAX, EAX);
 				MOV32RtoM((uptr)&psxRegs.GPR.r[_Rt_], EAX);
 #ifndef __WIN32__
@@ -1486,7 +1498,10 @@ static void recLW() {
 					if (!_Rt_) return;
 					iRegs[_Rt_].state = ST_UNK;
 
-					CALLFunc((uptr)GPU_readData);
+					//CALLFunc((uptr)GPU_readData);
+					MOV64ItoR(RAX, (uptr)GPU_readData);
+					CALL64R(RAX);
+
 					MOV32RtoM((uptr)&psxRegs.GPR.r[_Rt_], EAX);
 					return;
 
@@ -1494,7 +1509,10 @@ static void recLW() {
 					if (!_Rt_) return;
 					iRegs[_Rt_].state = ST_UNK;
 
-					CALLFunc((uptr)GPU_readStatus);
+					//CALLFunc((uptr)GPU_readStatus);
+					MOV64ItoR(RAX, (uptr)GPU_readStatus);
+					CALL64R(RAX);
+
 					MOV32RtoM((uptr)&psxRegs.GPR.r[_Rt_], EAX);
 					return;
 			}
@@ -1838,7 +1856,9 @@ static void recSH() {
 				}
 				//PUSHI  (addr);
 				MOV64ItoR(X86ARG1, addr);
-				CALLFunc  ((uptr)SPU_writeRegister);
+				//CALLFunc  ((uptr)SPU_writeRegister);
+				MOV64ItoR(RAX, (uptr)SPU_writeRegister);
+				CALL64R(RAX);
 #ifndef __WIN32__
 				//resp+= 8;
 #endif
@@ -1914,7 +1934,9 @@ static void recSW() {
 						//PUSH64M((uptr)&psxRegs.GPR.r[_Rt_]);
 						MOV32MtoR(X86ARG1, (uptr)&psxRegs.GPR.r[_Rt_]);
 					}
-					CALLFunc((uptr)GPU_writeData);
+					//CALLFunc((uptr)GPU_writeData);
+					MOV64ItoR(RAX, (uptr)GPU_writeData);
+					CALL64R(RAX);
 #ifndef __WIN32__
 					//resp+= 4;
 #endif
@@ -1928,7 +1950,9 @@ static void recSW() {
 						//PUSH64M((uptr)&psxRegs.GPR.r[_Rt_]);
 						MOV32MtoR(X86ARG1, (uptr)&psxRegs.GPR.r[_Rt_]);
 					}
-					CALLFunc((uptr)GPU_writeStatus);
+					//CALLFunc((uptr)GPU_writeStatus);
+					MOV64ItoR(RAX, (uptr)GPU_writeStatus);
+					CALL64R(RAX);
 #ifndef __WIN32__
 					//resp+= 4;
 #endif
