@@ -3090,21 +3090,14 @@ void SETE8R( x86IntRegType to ) { SET8R(0x94, to); }
 /* push imm32 */
 void PUSH32I( u32 from ) 
 {
-    X86_64ASSERT();
+    //X86_64ASSERT();  //becomes sign extended in x86_64
 	write8( 0x68 ); 
    write32( from ); 
 }
 
 #ifdef __x86_64__
 
-/* push imm32 */
-void PUSHI( u32 from ) 
-{
-	write8( 0x68 ); 
-   write32( from ); 
-}
-
-/* push r32 */
+/* push r64 */
 void PUSH64R( x86IntRegType from ) 
 {
     RexB(0,from);
@@ -3112,7 +3105,7 @@ void PUSH64R( x86IntRegType from )
 	write8( 0x50 | from );
 }
 
-/* push m32 */
+/* push m64 */
 void PUSH64M( uptr from ) 
 {
 	write8( 0xFF );
@@ -3251,9 +3244,8 @@ void LEA32RRtoR(x86IntRegType to, x86IntRegType from0, x86IntRegType from1)
 {
     RexRXB(0, to, from0, from1);
 	write8(0x8d);
-	if( (from0&7) == EBP || (from1&7) == EBP ) {
 
-		if( (from0&7) == EBP ) SWAP(from0, from1);
+    if( (from1&7) == EBP ) {
 		ModRM(1, to, 4);
 		ModRM(0, from0, from1);
 		write8(0);
