@@ -978,6 +978,32 @@ char *               pCaptionText;
 
 static int fx=0;
 
+
+static Atom xv_intern_atom_if_exists( Display *display, char const * atom_name )
+{
+  XvAttribute * attributes;
+  int attrib_count,i;
+  Atom xv_atom = None;
+
+  attributes = XvQueryPortAttributes( display, xv_port, &attrib_count );
+  if( attributes!=NULL )
+  {
+    for ( i = 0; i < attrib_count; ++i )
+    {
+      if ( strcmp(attributes[i].name, atom_name ) == 0 )
+      {
+        xv_atom = XInternAtom( display, atom_name, False );
+        break; // found what we want, break out
+      }
+    }
+    XFree( attributes );
+  }
+
+  return xv_atom;
+}
+
+
+
 // close display
 
 void DestroyDisplay(void)
@@ -1010,7 +1036,7 @@ void DestroyDisplay(void)
 	shmdt(shminfo.shmaddr);
 	shmctl(shminfo.shmid,IPC_RMID,NULL);
 
-  Atom atom_vsync = XInternAtom(display, "XV_SYNC_TO_VBLANK", False);
+  Atom atom_vsync = xv_intern_atom_if_exists(display, "XV_SYNC_TO_VBLANK");
   if (atom_vsync != None) {
 	XvSetPortAttribute(display, xv_port, atom_vsync, xv_vsync);
   }
@@ -1084,6 +1110,9 @@ void CreateDisplay(void)
 
     exit(-1);
   }
+
+  depth = DefaultDepth(display, myscreen);
+
   for (i = 0; i < p_num_adaptors; i++) {
 	p_num_ports = ai[i].base_id + ai[i].num_ports;
 	for (p = ai[i].base_id; p < p_num_ports; p++) {
@@ -1097,16 +1126,22 @@ void CreateDisplay(void)
 				yuv_port = p;
 				yuv_id = fo[j].id;
 			}
-			if (fo[j].type == XvRGB && fo[j].depth >= 24)
+			if (fo[j].type == XvRGB && fo[j].bits_per_pixel == 32)
 			{
 				xv_port = p;
 				xv_id = fo[j].id;
 				xv_depth = fo[j].depth;
+				printf("RGB mode found.  id: %x, depth: %d\n", xv_id, xv_depth);
 
-				//break out of loops
-				j = formats;
-				p = p_num_ports;
-				i = p_num_adaptors;
+				if (xv_depth != depth) {
+					printf("Warning: Depth does not match screen depth (%d)\n", depth);
+				}
+				else {
+					//break out of loops
+					j = formats;
+					p = p_num_ports;
+					i = p_num_adaptors;
+				}
 			}
 		}
 		if (fo)
@@ -1127,25 +1162,31 @@ void CreateDisplay(void)
 	xv_port = yuv_port;
 	xv_id = yuv_id;
   }
+  else if (xv_depth && xv_depth != depth && yuv_port != -1)
+  {
+	use_yuv = 1;
+	printf("Acceptable RGB mode not found.  Using YUV.\n");
+	xv_port = yuv_port;
+	xv_id = yuv_id;
+  }
 
-  Atom atom_vsync = XInternAtom(display, "XV_SYNC_TO_VBLANK", False);
+  Atom atom_vsync = xv_intern_atom_if_exists(display, "XV_SYNC_TO_VBLANK");
   if (atom_vsync != None) {
 	XvGetPortAttribute(display, xv_port, atom_vsync, &xv_vsync);
 	XvSetPortAttribute(display, xv_port, atom_vsync, 0);
   }
 
- myvisual = 0;
- xv_depth = DefaultDepth(display, myscreen);
+myvisual = 0;
 
-  if(XMatchVisualInfo(display,myscreen, xv_depth, TrueColor, &vi))
-   {myvisual = &vi;depth=xv_depth;}
+if(XMatchVisualInfo(display,myscreen, depth, TrueColor, &vi))
+	myvisual = &vi;
 
- if(!myvisual)
-  {
-   fprintf(stderr,"Failed to obtain visual!!!\n");
-   DestroyDisplay();
-   return;
-  }
+if (!myvisual)
+{
+	fprintf(stderr,"Failed to obtain visual!\n");
+	DestroyDisplay();
+	return;
+}
 
 /*
  if(myvisual->red_mask==0x00007c00 &&
@@ -1369,8 +1410,15 @@ void BlitScreen32(unsigned char * surf,int32_t x,int32_t y)
      for(row=0;row<dx;row++)
       {
        lu=*((uint32_t *)pD);
-	destpix[row] = 
-          0xff000000|(RED(lu)<<16)|(GREEN(lu)<<8)|(BLUE(lu));
+
+#ifdef __BIG_ENDIAN__
+	if (surf == shminfo.shmaddr)
+		PUTLE32( &destpix[row], 0xff000000|(RED(lu)<<16)|(GREEN(lu)<<8)|(BLUE(lu)) );
+	else
+		destpix[row] = 0xff000000|(RED(lu)<<16)|(GREEN(lu)<<8)|(BLUE(lu));
+#else
+	destpix[row] = 0xff000000|(RED(lu)<<16)|(GREEN(lu)<<8)|(BLUE(lu));
+#endif
        pD+=3;
       }
     }
@@ -1384,8 +1432,14 @@ void BlitScreen32(unsigned char * surf,int32_t x,int32_t y)
      for(row=0;row<dx;row++)
       {
        s=GETLE16(&psxVuw[startxy++]);
-	destpix[row] = 
-        (((s<<19)&0xf80000)|((s<<6)&0xf800)|((s>>7)&0xf8))|0xff000000;
+#ifdef __BIG_ENDIAN__
+	if (surf == shminfo.shmaddr)
+		PUTLE32( &destpix[row], (((s<<19)&0xf80000)|((s<<6)&0xf800)|((s>>7)&0xf8))|0xff000000 );
+	else
+		destpix[row] = (((s<<19)&0xf80000)|((s<<6)&0xf800)|((s>>7)&0xf8))|0xff000000;
+#else
+	destpix[row] = (((s<<19)&0xf80000)|((s<<6)&0xf800)|((s>>7)&0xf8))|0xff000000;
+#endif
       }
     }
   }
@@ -1424,8 +1478,7 @@ void BlitToYUV(unsigned char * surf,int32_t x,int32_t y)
 	U = min(abs(R * -1214 + G * -2384 + B * 3598 + 4096 + 1048576) >> 13, 240);
 	V = min(abs(R * 3598 + G * -3013 + B * -585 + 4096 + 1048576) >> 13, 240);
 
-	destpix[row] = 
-        Y << 24 | V << 16 | Y << 8 | U;
+	PUTLE32(&destpix[row], Y << 24 | V << 16 | Y << 8 | U);
        pD+=3;
       }
     }
@@ -1448,8 +1501,7 @@ void BlitToYUV(unsigned char * surf,int32_t x,int32_t y)
 	U = min(abs(R * -1214 + G * -2384 + B * 3598 + 4096 + 1048576) >> 13, 240);
 	V = min(abs(R * 3598 + G * -3013 + B * -585 + 4096 + 1048576) >> 13, 240);
 
-	destpix[row] = 
-        Y << 24 | V << 16 | Y << 8 | U;
+	PUTLE32(&destpix[row], Y << 24 | V << 16 | Y << 8 | U);
       }
     }
   }
@@ -1479,7 +1531,7 @@ void RGB2YUV(uint32_t *s, int width, int height, uint32_t *d)
 
 			Y2 = min(abs(R * 2104 + G * 4130 + B * 802 + 4096 + 131072) >> 13, 235);
 
-			*d = U | Y1 << 8 | V << 16 | Y2 << 24;
+			PUTLE32(d, U | Y1 << 8 | V << 16 | Y2 << 24);
 			d++;
 		}
 	}
